#include <stdlib.h>

// Konstanten sind unveränderlich in ihrem Gültigkeitsbereich.
// Die Zuweisung einer konstanten Variablen ist ungültig.
// Die Speicherbereiche auf die ein Zeiger zeigt können auch als konstant gekennzeichnet werden.
//
// Um Warnungen bei der Zuweisung von konstanten Zeichenketten anzuzeigen muss -Wwrite-strings
// verwendet werden.
// gcc -Wall -o 11-const 11-const.c -std=c99 -lm -ggdb -Wwrite-strings

//  Eine normale Funktion, welche einen String verändert.
void testStrFunc(char * str){
  str[0] = 0;
}

// Eine Funktion welche einen String als Parameter nimmt, dieser ist aber unveränderlich.
void testStrConst(char const * str){
  str[0] = 0; // Der Kompiler wirft einen Fehler.
}

// Am besten man verwendet "const" immer sofern es möglich ist,
// somit weiß ein Programmierer, was er zu erwarten hat, wenn er eine Funktion aufruft...
// im letzen Fall ist klar, das str nicht veränderbar ist.

int main(){
	int * b = NULL; // b ist ein Zeiger auf int
	int const c = 4; // c ist ein konstanter int
	int const *f = NULL; // f ist ein Zeiger auf einen konstanten int
	int * const a = 5; // a ist ein konstanter Zeiger auf einen int
	int * const * e; // e ist ein Zeiger auf einen konstanten Zeiger auf einen int

	a = b; // keine gültige Zuweisung, da a ein konstanter Zeiger ist
	*f = 6; // keine gültige Zuweisung, da der Wert von f konstant ist
	f = 7; // gültige Zuweisung, allerdings wird dem Zeiger der Wert 7 zugewiesen
	*a = b; // gültige Zuweisung, allerdings wird der Wert von a durch die b (eine Adresse) ersetzt.
	c = 5; // keine gültige Zuweisung, da c konstant ist

	e = & b; // gültige Zuweisung, da der erste Zeiger, der auf einen konstanten Zeiger zeigt, umgebogen wird
	         // hierdurch zeigt e NICHT mehr auf einen konstanten zeiger, d.h. der typ ist nicht mehr int * const *
	         // um dies zu verhindern, sollte beim zeiger verbiegen ein cast durchgeführt werden (int * const *)

	char * string = "test";
	testStrFunc(string); // wir übergeben einen Zeiger auf den String
	testStrConst(string);

	// Wir übergeben direkt einen String.
	// Sollte die Funktion testStrFunc die Zeichenkette modifizieren, so wird
	// das Verhalten typischerweise undefiniert, oftmals zu Programmabstürzen führen.
	testStrFunc ("test String");
	testStrConst("test String2");
}


