#include <stdlib.h>
#include <stdio.h>

/*
 * Eine Struktur erlaubt eine Gruppierung mehrerer Variablen als neuen Datentypen.
 * Die einzelnen Elemente der Struktur können hierbei über den Namen angesprochen werden.
 */

// struct ist das Schlüsselwort, welches auf die Struktur hinweist.
// Hier definieren wir die Struktur.
struct Person{
   int birthYear; // zwei Elemente, einen birthYear
   char * name;
};

// Aufbau der Struktur im Speicher ist sequentiell, d.h. zuerst steht birthYear, dann ein Zeiger auf
// einen Namen. Der Kompiler weiß über den Namen bescheid an welcher Stelle im Speicher die Objekte stehen.

/**
 * Die Funktion gibt die "Werte" einer Person aus.
 */
void printPerson(struct Person * p){
   // da p ein Zeiger auf eine Person ist müssten wir eigentlich für den Zugriff eines Elementes schreiben:
   // (*p).name
   // der "->" ist synonym für (*p). und erhöht die Lesbarkeit.

   printf("%s ist geboren im Jahr %d.\n", p->name, p->birthYear);
}

int main(){
  struct Person customer; // customer ist jetzt eine Variable vom Typ der Struktur

  // Zugriff auf die Elemente der Struktur bekommen wir mit "."
  customer.birthYear = 1998;

  // wir setzen den Namen auf einen String:
  customer.name = "hans müller";

  printPerson(& customer);


  // wir können Strukturen auch direkt initalisieren:
  struct Person franz = {
			  .name = "Franz" ,
			  .birthYear = 1980
			};

  printPerson(& franz);


  // wir können uns auch die Größe der Strukturen (bzw. beliebiger Datentypen) ausgeben lassen:
  // dies geschieht mit dem Compiler Befehl "sizeof"
  // der Rückgabewert ist vom Datentyp size_t, meistens "unsigned long"
  printf("Die Struktur von franz ist %zu Bytes groß\n", sizeof(franz) );

  // sizeof kann auch auf Datentypen direkt angewendet werden:
  printf("Ein integer ist %zu Bytes groß\n", sizeof(int) );
  printf("Ein long ist %zu Bytes groß\n", sizeof(long) );
  printf("Ein char* ist %zu Bytes groß\n", sizeof(char *) );
  printf("Ein char ist %zu Byte groß\n", sizeof(char) );

  // auf 64 Bit Systemen ist struct Person 16 Byte groß, integer ist 4 Byte, char * 8 Byte.
  // das kommt daher, dass der Compiler die Datentypen so im Speicher anordnet, dass der Zugriff optimal erfolgt, das sind typischerweise vielfache der Größe von Basisdatentypen bzw. Wortgrenzen.
  // da der Compiler die Reihenfolgen der Elemente einer Struktur nicht verändern kann, kann es sinnvoll
  // sein diese von Hand zu ordnen.
  // Ein Beispiel:
  struct Waste{
      char a;
      int i;
      char b;
      int j;
  };

  struct Besser{
      int i;
      int j;
      char a;
      char b;
  };

  printf("Die Struktur Waste ist %lu Bytes groß. Besser ist %lu Bytes groß.\n", sizeof(struct Waste), sizeof(struct Besser));

  // welche der beiden Strukturen sind also vorteilhaft um den Speicherverbrauch zu minimieren?

  return 0;
}
