#include <stdio.h>

void varTest(){
  int c = 1; // eine Variable hat einen Gültigkeitsbereich und eine Lebensdauer.
  // der Wert bleibt über die Lebensdauer erhalten und Speicher für die Variable zugewiesen.
  // nur innerhalb des Gültigkeitsbereiches kann die Variable mit dem Namen angesprochen werden.
  // Das liegt daran, das der Speicherbereich für die Variable nur für den Gültigkeitsbereich
  // gültig ist. (Später mehr).
  // Lebensbereiche werden hierarchisch in Blöcken gültig.

  {
    int d; // d ist nur innerhalb des Blockes bekannt.
  }


  printf("c ist %d\n", c);
  // Variablen können sich gegenseitig verdecken. Es gilt jeweils die letzte Definition.
  // um die Verdeckung zu sehen mit -Wshadow kompilieren
  // gcc -Wall -o 14-gueltigkeitsbereich 14-gueltigkeitsbereich.c -std=c99 -lm -ggdb -Wshadow
  {
    int c = 4;

    // die außere Variable mit dem Namen c ist immer noch existent, d.h. ihr Lebenszyklus ist noch nicht beendet.
    printf("c ist innen %d\n", c);

    {
      int c = 5;
      // die Tiefe der Verschachtelung ist hierbei beliebig.
    }
  }

  printf("c ist außen noch %d\n", c);

  // der Lebenszyklus von der Variablen "c" endet hier.
}

// globale Variable, diese Variable kann in einer weiteren
// C-Datei durch "extern int variable;" bekannt gemacht (deklariert) und dort genutzt werden.
// hier wird die Variable definiert, d.h. ein Speicherplatz zugewiesen & sie auch noch initalisiert!
int variable = 2;

// diese Variable ist eine "Modul"Variable und nur innerhalb dieser C-Datei unter dem Namen ansprechbar.
static int modulVar = 2;

// funktionsnamen können auch nur innerhalb des Modules zugreifbar sein, wenn man sie als static definiert.
// andernfalls sind die Funktionen auch in anderen hinzu gelinkten Objektdateien verwendbar!
static void test(){
}

void incVar(){
  variable++;
}


void printVar(){
  printf("Var is %d\n", variable);
}


// statische Variablen bekommen einen Speicherplatz zugewiesen, die Namen können aber auch nur innerhalb
// einer Funktion == BLOCK verwendet werden:
void staticFuncInc(){
// diese Variable wird zwar bei Programmstart initalisiert, ist aber nur innerhalb der Funktion bekannt.
  static int variable = 1;
  printf("Der statische Wert ist %d\n", variable);
  variable++;
}


int main(){

  varTest();

  printVar();
  incVar();
  printVar();

  staticFuncInc();
  staticFuncInc();

  return 0;
}
