/**
 * Der C-Präprozessor verändert den Source Code bevor dieser in Maschinencode umgewandelt wird.
 * Der Präprozessor kann mit Anweisungen gesteuert werden, diese beginnen mit "#".
 */

// diese Anweisung sorgt dafür, dass der Inhalt der Datei "stdlib.h" an diese Stelle kopiert wird.
// Der Präprozessor arbeitet rekursiv, d.h. wenn in dieser Datei weitere Dateien inkludiert werden,
// so werden sie ebenfalls an die entsprechende Stelle kopiert.
// zum Testen können wir den Befehl "cpp" auf die Quellcode Datei ausführen und sehen wie der
// Präprozessor den Code verändert hat.
#include <stdio.h>

// der Präprozessor kann Warnungen ausgeben:
#warning Bitte kompilieren Sie den Code mit "-D VARIABLE=\"WHATEVER\"" -DNAME=jens

// Bedingungen sind auch erlaubt:
#ifdef VARIABLE
// hier tun wir nichts:
#else
// der Kompiliervorgang kann auch mit einem Fehler abgebrochen werden:
#error Der Code muss mit -DVARIABLE="..." kompiliert werden!
#endif


int main(){
  // für VARIABLE setzt der Präprozessor hierbei die auf der Kommandozeile angebenen Werte ein.
  printf("Die Variable ist %s\n", VARIABLE);
  // es gibt noch diverse Makros für den Compiler und Präprozessor z.b. __FUNCTION__

  printf("Es wurde die Funktion %s aufgerufen\n", __FUNCTION__);

  // wir können auch neue Worte aneinanderreihen.
 char const * name = "test" VARIABLE;
 printf("Aneinandergereiht ist es %s\n", name);

  // wir können auch die Definition von Variablen ändern:
  // erstmal entfernen wir die Definition:
#undef VARIABLE

 // und dann definieren wir die Variable erneut, dies erfolgt analog zur Kommandozeile:
#define VARIABLE "HANS"
  printf("Die Variable ist umdefiniert zu %s\n", VARIABLE);

//#undef VARIABLE
 // mit # wird der übergebene Wert zu einem String umgewandelt.
//#define VARIABLE(x) #x
//  printf("Die Variable ist umdefiniert zu %s\n", VARIABLE(hans));


 // Makros, sind wie funktionen, der Code wird jedoch zur Laufzeit ersetzt:
 // wir ersetzen sqr(x) durch das quadrat.
#define sqr(x)   x*x

 // vorsicht ist bei Makros geboten, denn die Code ersetzung kann zu anderen Ergebnissen führen:
  printf("Sqr(4) = %d Sqr(Sqr(4)) = %d  Sqr(4+1) = %d\n", sqr(4), sqr(sqr(4)), sqr(4+1));
 // Der Kompiler hat sqr(4+1) ersetzt zu 4+1*4+1 = 9\n"
#undef sqr
 // auf korrekte Klammerung achten:
#define sqr(x) ((x)*(x))
  printf("Sqr(4+1) mit korrektem Makro ist %d\n", sqr(4+1));



 // Definierte Werte können auch mit existierenden Werten verbunden werden (token concatenation)
#define CONCAT(x) x##Name

 char * hansName  = "Hans Müller";
 char * franzName = "Fränzchen";
 char * jensName = "Jens Liebknecht";

 printf( "Franz Name ist %s\n", CONCAT(franz) );

 //leider funktioniert sowas nicht, da zunächst NAME mit Name konkateniert wird. NAME Ist dannach kein Makro mehr.
 // printf( "Der ausgewählte Name ist %s\n", CONCAT(NAME) );

 // Makros können schön für das Debugging genutzt werden
 // falls das Debugging aktiviert ist geben wir etwas aus:
#ifdef DEBUG
  printf("Das Debugging ist aktiviert!\n");
#endif

 // Man kann sich auch ein Makro dafür schreiben zur Vereinfachung:
#ifdef DEBUG
  #define debug(x) printf(x)
#else
  // falls das Debugging nicht aktiviert ist, so entfernen wir einfach den Aufruf von printf.
  #define debug(x)
#endif


 // diese Zeile wird nur ausgegeben falls beim Kompilieren DEBUG definiert war.
 debug("Das Debugging ist aktiviert, im Makro\n");


 debug("Die Funktion endet hier\n");


 // vorsicht, bei   Mehrzeiligen Makros signalisiert ein \ das das Makro noch nicht endet:
#define DOIT \
    printf("DOIT\n");

 // ohne \ würde der Code an dieser Stelle hier ausgeführt werden.

 // X-Macros, Makros rufen Makros auf, geht zu weit, siehe http://en.wikipedia.org/wiki/C_preprocessor

 return 0;
}
