/**
 In dieser Lektion werden wir mehr über das Speichermanagement lernen.
 */
#include <stdio.h>
#include <string.h>
#include <stdlib.h>

/**
  Wenn wir eine Funktion schreiben wollen die einen Zeiger auf etwas zurück gibt z.B. auf einen Array,
  dann müssen wir aufpassen, dass wir die Lebensdauer der Variablen einhalten.
 */
int * getValues(){
  int values[] = {1, 2, 3, 4, 5};
  return values;
  /**
   Dieser Code ist sehr gefährlich, denn der Array ist nur gültig innerhalb dieser Funktion.
   Der Zeiger wird zurück gegeben, u.U. ist der Array im Speicher längst überschrieben, wenn wir
   darauf zugreifen!  Daher kann dieser Code schwere Folgefehler erzeugen, welche nicht leicht aufzufinden sind.
  */
}

/*
Allgemein liegen alle definierten Variablen in einem bestimmten Speicher, dem Stack.
Dieser ist, wie der Name schon sagt, ebenso organisiert, d.h. wenn ein neuer Gültigkeitsbereich (Block) betreten wird, so werden die Variablen auf dem Stack untergebracht, verlässt man den Bereich, so
werden die Addressen für den nächsten "Block" gespeichert.
Funktionsaufrufe werden auch auf dem Stack unterbracht.

Wir können Speicher neben dem Stack auch auf dem Heap allokieren. Der Heap wird komplett vom
Benutzer verwaltet, dieser muss Speicher anfordern und auch wieder frei geben.
(Bei Programmende wird der Speicher automatisch freigegeben).
*/

/*
  Ein Beispiel, wie man Speicher anfordert und von einer Funktion zurück geben kann.
 */
float * allocateArray(int length){
  // malloc fordert einen Speicherblock von einer bestimmten Größe an, wir wollen length Elemente, diese sind jeweils so groß wie ein float. Der Compiler stellt den Befehl sizeof zur Verfügung um diese zu ermitteln.
  return malloc(length * sizeof(float));
}

int main(int argc, char ** argv){
  const long arraySize = atol(argv[1]);

  float * const array = allocateArray(arraySize);

  for(int i=0; i < arraySize; i++){
    array[i] = i;
  }

  // mit dem Befehl free geben wir den Speicher wieder für anderweitige Nutzung frei.
  free(array);

  printf("Completed!\n");

  /*
    Wenn man ein Programm schreibt, welches mittels Pointern auf Speicher zugreift können mehrere Fehler passieren:
    - man greift auf Speicher zu, welcher einem (vermutlich) nicht gehört:
  */
  array[arraySize * 10] = 0;

  // - es wird auf Speicher zugegriffen, welcher nur lesbar ist (Maschinencode z.B. steht auch irgendwo im Speicher).

  // es wird auf den eigenen Speicher zugegriffen, nur evtl. in anderen Variablen:
  array[-1] = 0;
  array[arraySize + 1] = 0;

  //  - man greift auf Speicher zu, welchen man bereits freigeben hat:
  array[0] = 1; // ist dieser Code hier gültig?

  // wenn man Glück hat, so stürzt das Program ab mit einem Speicherfehler (Segmentation Fault / Segfault).

  // wenn man Pech hat ändert man z.B. eine Schleifenvariable so ab, dass die Schleife nie mehr abbricht
  // die Fehlerauswirkungen sind sehr unterschiedlich und sehr schwer zu debuggen, da die Fehlerursache
  // in anderen Routinen als die Fehlerwirkung auftreten.
  // Die Fehlerwirkung kann auch bei manchen Läufen auftreten, bei anderen nicht, da die Speicherbereiche zufällig verteilt werden.

  // ein Werkzeug um Speicherfehler zu debuggen ist Valgrind, dieses sollte man sich unbedingt anschauen.
  // valgrind ./24-dynamischer-speicher
  // Vorsicht: durch Valgrind Tools verlängert sich die Laufzeit!

  // typischerweise sollte man stets so debuggen, das zunächst Valgrind verwendet wird, bei Fehlerfreiheit dann gdb (oder andere debugger).

  return 0;
}