/**
  @section DoxyBeispiel
  @brief DoxyGen Beispiel
  @file 26-doxygen.c
  @author Julian Kunkel
  @example

  Doxygen ist ein Werkzeug um Dokumentation im Code auszuwerten.
  http://www.stack.nl/~dimitri/doxygen/index.html

  Useful tips for doxygen in C files:
  http://svn.gnucash.org/docs/head/tipshints.html

  Zunächst muss eine Konfigurationsdatei erzeugt werden:
    doxygen -g

  Im Beispiel wurde diese bereits angepasst. Die Dokumentation wird erstellt mittels:
  doxygen 26-Doxyfile

  Doxygen ist im Speziellen auch für C++ sinnvoll um die Klassen zu dokumentieren.

  Alle Deklarationen einer Funktion, Typedefs, Struct oder Enums sollten dokumentiert werden.
  Normalerweise werden Header Dateien (*.h) im Detail dokumentiert, da diese die Schnittstellen definieren. In den dazu gehörigen *.c Dateien werden dann die Funktionsdefinitionen nicht mehr dokumentiert (reduziert Redundanz).
*/


/**
 Die Variable enthält einen Zähler für die Anzahl der Zugriffe...
*/
int myGlobalVariable;

/**
  Dieser Teststruct enthält doxygen Dokumentation.
*/
struct myTestStruct{
  /**
    The number of the test struct.
  */
  int myNumber;

  /**
    The name of the structure.
  */
  char * myName;
};

/**
  Eine Interne, nur im Modul verfügbare Datei wird normalerweise nicht automatisch in die Doku übernommen.
*/
static void myTestFunc(){
  myGlobalVariable = 4;
  return;
}

/**
  Hier erzeugen wir einen Callergraph, d.h. wer alles diese Funktion aufruft.
  Dies wird automatisch gemacht, sofern die CALL_GRAPH bzw. CALLER_GRAPH Option im Doxyfile gesetzt ist.
  @callergraph
*/
void myTest(){
  myGlobalVariable = 4;
  return;
}



// Die Dokumentation einer Funktion muss am Protypen erfolgen!
/**
  Main routine -- invoked upon startup of the executable.

  @param argc The number of arguments.
  @param argv Array of parameter strings.
  @return Exit code of the application.
*/
int main(int argc, char ** argv){
   myTestFunc();
   myTest();
   myGlobalVariable = 1;
   return 0;
}
