/**
 * Julian's Code für Julia-Mengen - see http://de.wikipedia.org/wiki/Julia-Menge ;-)
 * (c) Julian Kunkel 2010
 *
 * Licensed under BSD License...
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <string.h>
#include <assert.h>
#include <math.h>

#include <time.h>

static void writeMatrix(int width, int height, int ** matrix, char * file, int maxIter, int colorSchema, int COLORMAX );

int main(int argC, char ** argV){
    if (argC != 5){
	printf("Syntax: <Program> <Value> <ImageFileName> <ColorSchema> <size>\n");
	printf(" ColorSchema: <R|G|B> = 2 | 4 | 8  i.e. value of 10 is Purple\n");
	printf("              add 1 to use single color, otherwise multicolor\n");
	exit(1);
    }

    double cVal = atof(argV[1]);
    char * file = argV[2];
    int colorSchema = atoi(argV[3]);
    int colormax =  256;
    int size = atoi(argV[4]);

    if (colorSchema & 1) {
      colormax =  256;
    }else{
      colormax = (256 * 256 * 256);
    }

    if(colorSchema == 0){
      printf("Error: colorSchema 0 does not make sense, I will use 14 instead\n");
      colorSchema = 14;
    }

    printf("Sin %f into file %s\n", cVal, file);

    // matrix enthält anzahl der Iterationen bis Julia Menge divergiert.
    int * matrixData = malloc(sizeof(int) * size * size);

    memset(matrixData, colormax, sizeof(int) * size * size);

    // allocate memory for rows:
    int ** matrix = malloc(sizeof(void *) * size);

    // set rows:
    for (int y = 0 ; y < size; y++){
      matrix[y] = matrixData + size * y;
    }

    struct timespec startTime;
    struct timespec endTime;

    clock_gettime(CLOCK_REALTIME, & startTime);

    // now walk through every point in x/y-space:
    for (int y = 0; y < size; y++){
	for (int x = 0; x < size; x++){
	  matrix[x][y] = (int) ( sin((( (double) x + y) * 360.0 * cVal / (size*size) )) * colormax + colormax ) / 2;
	}
    }
    // end computation

    clock_gettime(CLOCK_REALTIME, & endTime);

    printf("Time: %fs\n", endTime.tv_sec - startTime.tv_sec + (endTime.tv_nsec - startTime.tv_nsec) * 1e-9);

    // write Matrix
    writeMatrix(size, size, matrix, file, colormax, colorSchema, colormax);

    free(matrixData);

    return 0;
}



/*
 * The following code works and should not be modified...
 */
int write_bmp(const char *filename, int width, int height, char *rgb);

void writeMatrix( int width, int height, int ** matrix, char * file, int maxIter, int colorSchema, int colormax ){
  char * rgb = malloc(width*height*3);

  double part = ( colormax - 1.0 ) / maxIter;
  int colorMultiplier[] = { (colorSchema & 2) / 2, (colorSchema & 4) / 4, (colorSchema & 8) / 8 };

  // change matrix format and compute real values:
  for (int y = 0 ; y < height; y++){
    // much room for optimization here.
    for (int x = 0 ; x < width ; x++){
      int color = (int) (  matrix[x][y] * part - 0.01 );

      int pos = 3 * (y * width + x );

      if(  colorSchema & 1   ){
	assert(color >= 0 && color <= 255);

	rgb[pos] =  color *  colorMultiplier[0];
	rgb[pos + 1 ] =  color *  colorMultiplier[1];
	rgb[pos + 2 ] =  color *  colorMultiplier[2];
      }else{
	assert(color >= 0);
	assert(color <= colormax);
	#define shft(x) (( color & ( 1 << (x*3) )) >> (x *2) )
	rgb[pos] = shft(0)+ shft(1) + shft(2) + shft(3) + shft(4)+ shft(5)+ shft(6)+ shft(7) ;
	#undef shft
	#define shft(x) (( color & ( 1 << (x*3+1) )) >> (x *2 + 1) )
	rgb[pos + 1] = shft(0)+ shft(1) + shft(2) + shft(3) + shft(4)+ shft(5)+ shft(6)+ shft(7) ;
	#undef shft
	#define shft(x) (( color & ( 1 << (x*3+2) )) >> (x *2 + 2 ) )
	rgb[pos + 2] = shft(0)+ shft(1) + shft(2) + shft(3) + shft(4)+ shft(5)+ shft(6)+ shft(7) ;
	#undef shft
      }
    }
  }

  write_bmp(file, width, height, rgb);

  free(rgb);
}